      module flxacc

      contains


      subroutine readflux
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-11-12                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program reads accumulated values of            *
*                                                                  *
*     QUANTITY                     MARS ABBR    CODE               *
*                                                                  *
*     large scale precipitation       LSP       142                *
*     convective precipitation        CP        143                *
*     surface sensible heat flux      SSHF      146                *
*     surface solar radiation         SSR       176                *
*     u stress                        EWSS      180                *
*     v stress                        NSSS      181                * 
*                                                                  *
*                                                                  *
********************************************************************

       USE GRIB_API

      integer kunit(12),ierr,lenght,iword,i,j,itime
      integer itges,ijp
      integer ij,im,id,ih,idel,ind,ic,jmd,hms,jout,ipar,l
      integer iday,ihour,istep

      character*1 mode

      double precision jul,juldate


      real, allocatable, dimension(:,:,:,:) :: lsp,lspacc
      real, allocatable, dimension(:,:,:) :: lsp6
      real, allocatable, dimension(:,:,:) :: lsp9
      real, allocatable, dimension(:,:,:) :: lspi
      real, allocatable, dimension(:) :: values,steps

      integer,allocatable, dimension(:) :: idate
      integer,allocatable, dimension(:,:) :: igrib
      integer :: params(6),npar
      character*10,allocatable, dimension(:) :: fname

      NAMELIST /NAMFX2/
     1            NX, NY, 
     2            MAXTIME,JPOLY,
     3            JHRF,
     4            RLO0, RLA0, DX, DY

C
C Change directory info below, character length must
C be consistent with that above for fname1 etc.
C

      params=(/142,143,146,180,181,176/) !lsp,cp,sshf,ewss,nsss,ssr

      call posnam (4,'NAMFX2')
      read (4,NAMFX2)

      idimens=nx*ny
      npar=6

      allocate(fname(12))
      do i=1,12
        write(fname(i),'(a5,I2.2,a3)') 'surf_',i,'_ub'
      enddo

      allocate(lsp(nx,ny,jpoly,npar))
      allocate(lspacc(nx,ny,jpoly,npar))

      allocate(lspi(nx,ny,npar))

      allocate(idate(maxtime+2))

      allocate(steps(6))



      steps(1:2)=jhrf/1000.  ! rain [m->mm]
      steps(3:6)=3600.*jhrf  ! other fluxes [J/m^2 ->W/m^2]

      mode='D'
        l=0
        do i=0,12,jhrf
          if(i >0) then
            call grib_open_file(kunit(i),fname(i),'r')
            call grib_count_in_file(kunit(i),n)
          endif
        enddo
        allocate(igrib(npar,1))

        ll=1
        do ktime=1,maxtime/12  ! Loop over days
        do itime=jhrf,12,jhrf     ! Loop over steps (1,3,6)
        lpar:        do j=1,npar
            i=1
            
            call grib_new_from_file(kunit(itime),igrib(j,i), iret)
            
            if(iret == -1) then
              exit lpar
            endif

            call grib_get(igrib(j,i),"paramId",ipar)
            if(ipar .ne. params(j)) then
              STOP 'unexpected parameter'
            endif
            call grib_get(igrib(j,i),"date",iday)
            call grib_get(igrib(j,i),"time",ihour)
            call grib_get(igrib(j,i),"stepRange",istep)

            jul=juldate(iday,ihour*100)
            jul=jul+dble(istep)/dble(24.0)
            call caldate(jul,jmd,hms)
            idate(ll)=jmd*100+hms/10000


            call grib_get_size(igrib(j,i),'values',numberOfValues)
*          write(*,*) 'numberOfValues=',numberOfValues
 
            allocate(values(numberOfValues), stat=iret)
      !     get data values
          call grib_get(igrib(j,i),'values',values)
          lla=ll
          if(lla>4) lla=4
          do k=1,ny
            do l=1,nx
            lspacc(l,k,lla,j)=values(nx*(ny-k)+l)/steps(j)
            enddo
          enddo

!Flux from initial step or difference between accumulated fluxes           
          if(mod(istep,12) == jhrf) then
            lsp(:,:,lla,j)=lspacc(:,:,lla,j)
          else
            lsp(:,:,lla,j)=lspacc(:,:,lla,j)-lspacc(:,:,lla-1,j)
          endif

!          write(*,*) j,ipar,iday,ihour,istep,lla,lsp(1,1,lla,j)
        enddo lpar

        if(iret == -1) exit

*
      if(ll .gt. 3) then
* Interpolate deaccumulated fluxes with a cubic-polynomial solution
* or with a modified linear solution for rainfall
      do i=1,2
        call darain(lsp(:,:,:,i),lspi(:,:,i),nx,ny,jpoly)
      enddo
      do i=3,npar
        call dapoly(lsp(:,:,:,i),lspi(:,:,i),nx,ny,jpoly)
      enddo
          write(*,*) 'Int:',lspi(1,1,:)

*
* Write out interpolated / deacc. fields at this timestep
      call writeout(itime,igrib,
     2     lspi,params,idate(ll-2),
     3     nx,ny,maxtime,idimens,npar)

C *** Shift flux fields back one unit in time
      do i=1,npar
        call daback(lspacc(:,:,:,i),nx,ny,jpoly)
        call daback(lsp(:,:,:,i),nx,ny,jpoly)
      enddo

      else
        if(ll .eq. 3) then
*
* Write out first field (uninterpolated)
      call writeout(itime,igrib,
     2     lsp(:,:,1,:),params,idate(ll-2),
     3     nx,ny,maxtime,idimens,npar)
        endif
      endif

      ll=ll+1

      enddo ! itime
      enddo ! ktime

      return
      end subroutine readflux


      subroutine writeout(itime,igrib,
     2     lspi,params,idate,
     3     nx,ny,maxtime,idimens,npar)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-11-12                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This subroutine writes out the deaccumulated and    *
*              interpolated ECMWF Flux quantities to a grib file   *
*              for every time/date.                                *
*              Note: Negative values are set to zero for all fields*
*              except SSHF                                         *
*                                                                  *
*              filename:  fluxjjjjmmddhh                           *
*                                                                  *
********************************************************************

      use GRIB_API 

      integer i,j,jout,npar
      integer idatehelp,itime
      character date*10,filename*48,mode*1,syscom*80
      integer lunit,ierr      
      character*30 fname
 
      real lspi(nx,ny,npar),values(nx*ny)
      integer idate,params(npar)
      integer igrib(npar,1)

      mode='C'
*
      write(date,'(i10.10)') idate
C
C Change the following directory info
C Character length must be consistent with declaration
C for "filename" above
C
      filename='flux'//date

      call grib_open_file(LUNIT,filename,'w')
         do n=1,npar

           call grib_set(igrib(n,1),"date",idate/100)
           call grib_set(igrib(n,1),"time",mod(idate,100)*100)
           call grib_set(igrib(n,1),"step",0)

           call grib_set(igrib(n,1),"paramId",params(n))
          do k=1,ny
            do l=1,nx
              values(nx*(ny-k)+l)=lspi(l,k,n)
            enddo
          enddo
          call grib_set(igrib(n,1),"values",values)

           call grib_write(igrib(n,1),Lunit)
         enddo
      call grib_close_file(LUNIT)


      return
      end  subroutine writeout

      subroutine dapoly(xfeld,sfeld,nx,ny,jpoly)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-03-29                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program interpolates deaccumulated fluxes of   *
* an ECMWF model FG field using a cubic polynomial solution which  *
* conserves the integrals of the fluxes within each timespan       *
* The interpolated values are output at the central point of the 4 *
* accumulation timespans                                           *
*                                                                  *
********************************************************************

      integer i,j
      real xfeld(nx,ny,jpoly),sfeld(nx,ny)
      real pya,pyb,pyc,pyd,xa,xb,xc,xd

      do 400 i=1,nx
      do 400 j=1,ny
         xa=xfeld(i,j,1)
         xb=xfeld(i,j,2)
         xc=xfeld(i,j,3)
         xd=xfeld(i,j,4)
         pya=(xd-xa+3.*(xb-xc))/6.
         pyb=(xc+xa)/2.-xb-9.*pya/2.
         pyc=xb-xa-7.*pya/2.-2.*pyb
         pyd=xa-pya/4.-pyb/3.-pyc/2.
         sfeld(i,j)=8.*pya+4.*pyb+2.*pyc+pyd
400   continue

      return
      end subroutine dapoly

      subroutine daback(xfeld,nx,ny,jpoly)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-08-11                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************

      integer i,j
      real xfeld(nx,ny,jpoly)

      do 400 i=1,nx
      do 400 j=1,ny
         xfeld(i,j,1)=xfeld(i,j,2)
         xfeld(i,j,2)=xfeld(i,j,3)
         xfeld(i,j,3)=xfeld(i,j,4)
400   continue

      return
      end  subroutine daback

      subroutine darain(xfeld,sfeld,nx,ny,jpoly)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-04-10                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program interpolates deaccumulated fluxes of   *
* an ECMWF FG rainfall field using a modified linear solution.     *
* The interpolated values are output at the central point of the 4 *
* accumulation timespans                                           *
*                                                                  *
********************************************************************

      integer i,j
      real xfeld(nx,ny,jpoly),sfeld(nx,ny)
      real xa,xb,xc,xd,xac,xbd

      do 400 i=1,nx
      do 400 j=1,ny
         xa=xfeld(i,j,1)
         xb=xfeld(i,j,2)
         xc=xfeld(i,j,3)
         xd=xfeld(i,j,4)
         if (xa.lt.0.) xa=0.
         if (xb.lt.0.) xb=0.
         if (xc.lt.0.) xc=0.
         if (xd.lt.0.) xd=0.
         xac=0.5*xb
         if ((xa+xc).gt.0.) xac=xb*xc/(xa+xc)
         xbd=0.5*xc
         if ((xb+xd).gt.0.) xbd=xb*xc/(xb+xd)
         sfeld(i,j)=xac+xbd
400   continue

      return
      end subroutine darain


      end module flxacc
